/***********************************************************
 * binsearch_helper.c
 *   Used by binsearch.m
 *   Given a sorted array of doubles, 'array', and a double 
 %   scalar, 'key', returns a 2-vector containing the first
 *   and last index at which 'key' appears in 'array'.
 *   If 'key' does not appear, [0,0] is returned.
 * 
 * Author: David Ross, Oct. 2006
 * $Id$
 ***********************************************************/

#include "mex.h"
#include <stdlib.h>
#define NDEBUG
#include <assert.h>
#define LIST_TYPE double

int binsearch(LIST_TYPE *list, int len, LIST_TYPE key);
int lsearch(LIST_TYPE *list, int len, int start, LIST_TYPE key);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, 
		 const mxArray *prhs[]) {
   int Nx, Ny;
   double *array, key, *result;
   int start, stop;

   /* check for the proper number of arguments */
   if (nrhs != 2) {
      mexErrMsgTxt("wrong number of input arguments");
   }
   if (nlhs != 1) {
      mexErrMsgTxt("wrong number of output arguments");
   }

   /* get size of array */
   Ny = mxGetM(prhs[0]);
   Nx = mxGetN(prhs[0]);

   /* make sure the array is okay */
   if (!mxIsDouble(prhs[0]) || mxGetM(prhs[0]) < 1 || 
       mxGetN(prhs[0]) < 1) {
       mexErrMsgTxt("array must be double, with numel >= 1 ");
   }

   /* get array */
   array = mxGetPr(prhs[0]);

   /* make sure the key is okay */
   if (!mxIsDouble(prhs[1]) || mxGetM(prhs[1]) != 1 || 
       mxGetN(prhs[1]) != 1) {
       mexErrMsgTxt("key must be a scalar double");
   }

   /* get the search key */
   key = mxGetScalar(prhs[1]);

   /* create the result */
   plhs[0] = mxCreateDoubleMatrix(1, 2, mxREAL);
   result = mxGetPr(plhs[0]);
   
   start = binsearch(array, Nx*Ny, key);

   if (start < 0) {
      result[0] = 0.0;
      result[1] = 0.0;
   } else {
      stop = lsearch(array, Nx*Ny, start, key);
      result[0] = start+1.0;
      result[1] = stop+1.0;
   }
}

int binsearch(LIST_TYPE *list, int len, LIST_TYPE key) {
   int start = 0;
   int stop = len-1;
   int mid = start + ((stop-start) >> 1);

   assert(list != NULL && len > 0);

   while (start != stop) {
      if (list[mid] < key) {
	 start = mid+1;
      } else {
	 stop = mid;
      }
      mid = start + ((stop-start) >> 1);
   }
   return (list[start] == key) ? start : -1;
}

int lsearch(LIST_TYPE *list, int len, int start, LIST_TYPE key) {
   int ii;

   assert(list != NULL);
   assert(start >= 0 && start < len);

   for(ii = start+1; (ii < len) && (list[ii] == key); ii++);
   return ii-1;
}
